==> sapdb-source-7.3.00.48.tgz: V73_00_48/SAPDB_ORG/sys/src/SAPDB/Trace/Trace_SlotPool.hpp <==
/*!**************************************************************************

  module      : Trace_SlotPool.hpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  special area:
  description : Implementation of Kernel VTrace

  last changed: 2000-11-27  18:15
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


#ifndef TRACE_SLOT_POOL_HPP
#define TRACE_SLOT_POOL_HPP



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "gsp00.h"	// PASCAL: SP_basic_constants_and_types
#include "heo57.h"	// RTE   : Vmalloc/Vmfree
#include <new.h>    // SYSTEM: required for placement new


#include "SAPDB/Trace/Trace_Slot.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

class Trace_SlotPool
{
	friend class Trace_SlotPoolIterator;
	
public:
	
	Trace_SlotPool();
	
	bool Initialize( tsp00_Int4 UKTId);
	
	
	Trace_Slot & Get( tsp00_Int4 UKTId ) const;
		
private:
	
	tsp00_BytePtr	m_pSlotPool;
	tsp00_Int4      m_FirstSlot;
	tsp00_Int4		m_LastSlot;
};

/*---------------------------------------------------------------------------*/

class Trace_SlotPoolIterator
{
public:

    Trace_SlotPoolIterator( Trace_SlotPool &SlotPool );


	void operator ++();

	Trace_Slot & operator *() const;

    void Begin();
    
    bool End() const;

private:

    tsp00_Int4		m_CurrentSlot;
    Trace_SlotPool	&m_SlotPool;
};

/*===========================================================================*
 *  IMPLEMENTATION OF METHODS                                                *
 *===========================================================================*/

inline
Trace_SlotPool::Trace_SlotPool()
:
m_pSlotPool( NULL ),
m_FirstSlot (0 ),
m_LastSlot( 0 )
{
};

/*---------------------------------------------------------------------------*/

inline bool
Trace_SlotPool::Initialize( tsp00_Int4 NumUKTs )
{
	const	tsp00_Int4	ObjSize = Trace_Slot::GetSize();
			tsp00_Bool	bOkay	= FALSE;
			void		*pObj	= NULL;


	vmalloc( ObjSize * NumUKTs, &pObj, &bOkay );
	
	if (TRUE != bOkay) return( false );

	m_pSlotPool = (tsp00_BytePtr) pObj;
	m_FirstSlot = 1;
	m_LastSlot  = NumUKTs;

	// create trace_slots for each UKT via placement new

	for( tsp00_Int4 iUKT = 1; iUKT <= NumUKTs; iUKT++ )
	{
		new(( m_pSlotPool + (( iUKT-1 ) * ObjSize ))) Trace_Slot( iUKT );
	}
	
	// Initialize all trace_slots
	Trace_SlotPoolIterator SlotIter( *this );
	
	for( SlotIter.Begin(); !SlotIter.End(); ++SlotIter )
	{
		if (! ( *SlotIter ).Initialize() ) return( false );
	}
		
	return( true );
};

/*---------------------------------------------------------------------------*/

inline Trace_Slot &
Trace_SlotPool::Get
(
 tsp00_Int4 UKTId
 ) const
{
	return *( REINTERPRET_CAST( Trace_SlotPtr, 
		m_pSlotPool + (( UKTId-1 ) * Trace_Slot::GetSize())));
};

/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

inline
Trace_SlotPoolIterator::Trace_SlotPoolIterator
( 
 Trace_SlotPool &SlotPool 
 ) 
 : m_CurrentSlot( 1 ),
 m_SlotPool ( SlotPool )
{
};

/*---------------------------------------------------------------------------*/

inline void 
Trace_SlotPoolIterator::operator ++()
{
    ++m_CurrentSlot;
};

/*---------------------------------------------------------------------------*/

inline Trace_Slot &
Trace_SlotPoolIterator::operator *() const
{
    return m_SlotPool.Get( m_CurrentSlot );
};

/*---------------------------------------------------------------------------*/
 
inline void 
Trace_SlotPoolIterator::Begin()
{
    m_CurrentSlot = m_SlotPool.m_FirstSlot;
};

/*---------------------------------------------------------------------------*/

inline bool 
Trace_SlotPoolIterator::End() const
{
    return( m_CurrentSlot > m_SlotPool.m_LastSlot );
};

#endif  /* TRACE_SLOT_POOL_HPP */

==> sapdb-source-7.4.03.32.tgz: V74_03_32/SAPDB_ORG/sys/src/SAPDB/Trace/Trace_SlotPool.hpp <==
/*!**************************************************************************

  module      : Trace_SlotPool.hpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  special area:
  description : Implementation of Kernel VTrace

  last changed: 2000-11-27  18:15
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


#ifndef TRACE_SLOT_POOL_HPP
#define TRACE_SLOT_POOL_HPP



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "gsp00.h"  // PASCAL: SP_basic_constants_and_types

#include "RunTime/MemoryManagement/RTEMem_Allocator.hpp"
#include "SAPDBCommon/MemoryManagement/SAPDBMem_NewDestroy.hpp"
#include "SAPDBCommon/SAPDB_Types.hpp"
#include "Trace/Trace_Slot.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

class Trace_SlotPool
{
    friend class Trace_SlotPoolIterator;
    
public:
    
    Trace_SlotPool();
    
    bool Initialize( SAPDB_Int4 UKTId);
    
    
    Trace_Slot & Get( SAPDB_Int4 UKTId ) const;
        
private:
    
    Trace_SlotPtr   m_pSlot;
    SAPDB_Int4      m_FirstSlot;
    SAPDB_Int4      m_LastSlot;
};

/*---------------------------------------------------------------------------*/

class Trace_SlotPoolIterator
{
public:

    Trace_SlotPoolIterator( Trace_SlotPool &SlotPool );


    void operator ++();

    Trace_Slot & operator *() const;

    void Begin();
    
    bool End() const;

private:

    SAPDB_Int4      m_CurrentSlot;
    Trace_SlotPool  &m_SlotPool;
};

/*===========================================================================*
 *  IMPLEMENTATION OF METHODS                                                *
 *===========================================================================*/

inline
Trace_SlotPool::Trace_SlotPool()
:
m_pSlot( NULL ),
m_FirstSlot (0 ),
m_LastSlot( 0 )
{
};

/*---------------------------------------------------------------------------*/

inline bool
Trace_SlotPool::Initialize( SAPDB_Int4 NumUKTs )
{
    // create trace_slots for given UserKernelThreads via 
    // placement new and call default constructor.

    newarray (m_pSlot, NumUKTs, RTEMem_Allocator::Instance());

    if ( NULL == m_pSlot ) return( false );

    m_FirstSlot = 1;
    m_LastSlot  = NumUKTs;

    // Initialize all trace_slots
    SAPDB_Int4              iUKT;
    Trace_SlotPoolIterator  SlotIter( *this );
    
    for( SlotIter.Begin(), iUKT = 1; !SlotIter.End(); ++SlotIter, ++iUKT )
    {
        if (! ( *SlotIter ).Initialize( iUKT ) ) return( false );
    }
        
    return( true );
};

/*---------------------------------------------------------------------------*/

inline Trace_Slot &
Trace_SlotPool::Get
(
 SAPDB_Int4 UKTId
 ) const
{
    return( m_pSlot[ UKTId - 1 ] );
};

/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

inline
Trace_SlotPoolIterator::Trace_SlotPoolIterator
( 
 Trace_SlotPool &SlotPool 
 ) 
 : m_CurrentSlot( 1 ),
 m_SlotPool ( SlotPool )
{
};

/*---------------------------------------------------------------------------*/

inline void 
Trace_SlotPoolIterator::operator ++()
{
    ++m_CurrentSlot;
};

/*---------------------------------------------------------------------------*/

inline Trace_Slot &
Trace_SlotPoolIterator::operator *() const
{
    return m_SlotPool.Get( m_CurrentSlot );
};

/*---------------------------------------------------------------------------*/
 
inline void 
Trace_SlotPoolIterator::Begin()
{
    m_CurrentSlot = m_SlotPool.m_FirstSlot;
};

/*---------------------------------------------------------------------------*/

inline bool 
Trace_SlotPoolIterator::End() const
{
    return( m_CurrentSlot > m_SlotPool.m_LastSlot );
};

#endif  /* TRACE_SLOT_POOL_HPP */
